//  signal.h
//
//  ===== seoncd draft of project 1  Signal object =====
// 
//  By GCS for CSE 232 Fall 2006     4 Sep 2006

#ifndef SIGNAL_
#define SIGNAL_

using namespace std;
#include <cassert>       // for access to assert(exp) for error checking
#include <iostream>
#include <string>
#include <fstream>
#include <iomanip> 

const int maxSize = 1000;    // max size of any signal; change as needed 

/*   ====== used for plotting/visualizing a signal on cout ========
    output 1 line of char such as "           *             "
    Ncells is the number of characters of width for the plot
    typically 50 char of width, could be 10 or 100

    plotPoint  plots a single amplitude value amp across Ncells

    amp     in-only value to be plotted; must be between min and max
    Ncells  in-only number of char cells in char string
    min,max in-only min and max amplitudes of the signal
*/ 

void plotPoint ( double amp, int Ncells, double min, double max ); 


/*   =====  Plot an entire signal, one print line per amplitude

    ampValues  in-only array of size amplitude values
    size       in-only size of array
    LO,HI      in-only subscript range of active array
    Smin,Smax  in-only min and max of array
*/

void printPlot ( const double ampValues[], int size, int LO, int HI,
		 double Smin, double Smax ); 

//    compute and return the signal stats over subscript range [lo, hi]

void getStats(const double ampValues[], const int signalSize, 
	const int lo, const int hi,     // first 4 are in-only
	double& min, double&max,
	double& energy, double& mean);  // last 4 are out-only

//    ===== read in signal values from a named file  =====
//
//  filename     in-only name of file containing ASCII sequence of signal values
//  signal       out-only array to receive signal values
//  maxSize      in-only max size of signal array
//  actualSize   out-only size of signal read in

bool readSignal (const string& filename, double signal[], int maxSize, int& actualSize);


/*   ---------------------------- zmean ----------------------------------

     This function subtracts the current mean signal value from each signal
     amplitude, thus creating a signal of zero mean. Signal {1, 2, 3 } is
     transformed into  {-1, 0, 1}

*/

void zmean ( double Signal[],              //  in-only signal amplitudes
             const int signalSize,         //  in-only signal size
             const double currentMean      //  in-only current signal mean
            );

/*   ---------------------------- crop ------------------------------------

     This function crops the signal; it makes the current signal just those
     values of the input signal between the subscripts lo and hi. The size
     of the signal becomes (hi-lo+1). Any signal stats are thus invalid and
     must be recomputed. Cropping signal {1,3,5,7,9,11} at [1,4] resets the
     signal to {3,5,7,9}  
*/

void crop ( double Signal[],      // in-out signal amplitudes
            int& signalSize,      // in-out signal size
            const int lo,         // subscript of new first signal amplitude    
            const int hi          // subscript of new last signal value
          );

/*   ---------------------------- threshold --------------------------------

     Each signal amplitude s[j] is compared to the threshold t; if less than
     T, the amplitude is changed to 0.0, if not less than t, it is replaced
     with 1.0, thus a binary signal is returned.
*/

void threshold ( double Signal[],      //  in-out signal amplitudes
                 const int signalSize, //  in-only signal size, unchanged by op
                 const double t        //  in-only threshold value
               );
#endif


